// SPDX-License-Identifier: Unlicense
pragma solidity ^0.8.24;

import "./Reputation.sol";

struct Citation {
    int weightPercent;
    uint targetPostIndex;
}

struct Post {
    uint id;
    address sender;
    address author;
    string contentId;
    Citation[] citations;
}

contract Forum is Reputation {
    mapping(uint => Post) public posts;
    uint public postCount;

    event PostAdded(uint postIndex);

    function addPost(
        address author,
        string calldata contentId,
        Citation[] calldata citations
    ) external returns (uint postIndex) {
        postIndex = postCount++;
        Post storage post = posts[postIndex];
        post.author = author;
        post.sender = msg.sender;
        post.id = postIndex;
        post.contentId = contentId;
        for (uint i = 0; i < citations.length; i++) {
            post.citations.push(citations[i]);
        }
        emit PostAdded(postIndex);
    }

    function _onValidatePost(uint postIndex, uint amount) internal {
        _propagateValue(postIndex, int(amount));
    }

    function _propagateValue(uint postIndex, int amount) internal {
        Post storage post = posts[postIndex];
        int totalCitationWeight;
        for (uint i = 0; i < post.citations.length; i++) {
            totalCitationWeight += post.citations[i].weightPercent;
        }
        int totalOutboundAmount;
        for (uint i = 0; i < post.citations.length; i++) {
            int share = (amount * post.citations[i].weightPercent) / 100;
            totalOutboundAmount += share;
            _propagateValue(post.citations[i].targetPostIndex, share);
        }
        uint remaining = uint(amount - totalOutboundAmount);
        _update(address(this), post.author, remaining);
    }
}
